@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005, 2006, 2008-2010, 2012, 2014 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2006, 2008-2010, 2012, 2014, 2015 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
@@ -30,7 +30,7 @@
 // Copyright (C) 2012 Yi Yang <ahyangyi@gmail.com>
 // Copyright (C) 2012 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 // Copyright (C) 2012 Thomas Freitag <Thomas.Freitag@alfa.de>
-// Copyright (C) 2013, 2014 Jason Crain <jason@aquaticape.us>
+// Copyright (C) 2013-2015 Jason Crain <jason@aquaticape.us>
 // Copyright (C) 2014 Olly Betts <olly@survex.com>
 //
 // To see a description of the changes please see the Changelog file that
@@ -66,6 +66,7 @@
 #include <fofi/FoFiType1C.h>
 #include <fofi/FoFiTrueType.h>
 #include "GfxFont.h"
+#include "PSOutputDev.h"
 
 //------------------------------------------------------------------------
 
@@ -610,7 +611,7 @@ CharCodeToUnicode *GfxFont::readToUnicodeCMap(Dict *fontDict, int nBits,
   return ctu;
 }
 
-GfxFontLoc *GfxFont::locateFont(XRef *xref, GBool ps) {
+GfxFontLoc *GfxFont::locateFont(XRef *xref, PSOutputDev *ps) {
   GfxFontLoc *fontLoc;
   SysFontType sysFontType;
   GooString *path, *base14Name, *substName;
@@ -640,19 +641,19 @@ GfxFontLoc *GfxFont::locateFont(XRef *xref, GBool ps) {
 	case fontType1:
 	case fontType1C:
 	case fontType1COT:
-	  embed = globalParams->getPSEmbedType1();
+	  embed = ps->getEmbedType1();
 	  break;
 	case fontTrueType:
 	case fontTrueTypeOT:
-	  embed = globalParams->getPSEmbedTrueType();
+	  embed = ps->getEmbedTrueType();
 	  break;
 	case fontCIDType0C:
 	case fontCIDType0COT:
-	  embed = globalParams->getPSEmbedCIDPostScript();
+	  embed = ps->getEmbedCIDPostScript();
 	  break;
 	case fontCIDType2:
 	case fontCIDType2OT:
-	  embed = globalParams->getPSEmbedCIDTrueType();
+	  embed = ps->getEmbedCIDTrueType();
 	  break;
 	default:
 	  break;
@@ -669,7 +670,7 @@ GfxFontLoc *GfxFont::locateFont(XRef *xref, GBool ps) {
   }
 
   //----- PS passthrough
-  if (ps && !isCIDFont() && globalParams->getPSFontPassthrough()) {
+  if (ps && !isCIDFont() && ps->getFontPassthrough()) {
     fontLoc = new GfxFontLoc();
     fontLoc->locType = gfxFontLocResident;
     fontLoc->fontType = fontType1;
@@ -2206,30 +2207,14 @@ int GfxCIDFont::getNextChar(char *s, int len, CharCode *code,
 
   // horizontal
   if (cMap->getWMode() == 0) {
-    w = widths.defWidth;
+    w = getWidth(cid);
     h = vx = vy = 0;
-    if (widths.nExceps > 0 && cid >= widths.exceps[0].first) {
-      a = 0;
-      b = widths.nExceps;
-      // invariant: widths.exceps[a].first <= cid < widths.exceps[b].first
-      while (b - a > 1) {
-	m = (a + b) / 2;
-	if (widths.exceps[m].first <= cid) {
-	  a = m;
-	} else {
-	  b = m;
-	}
-      }
-      if (cid <= widths.exceps[a].last) {
-	w = widths.exceps[a].width;
-      }
-    }
 
   // vertical
   } else {
     w = 0;
     h = widths.defHeight;
-    vx = widths.defWidth / 2;
+    vx = getWidth(cid) / 2;
     vy = widths.defVY;
     if (widths.nExcepsV > 0 && cid >= widths.excepsV[0].first) {
       a = 0;
@@ -2533,13 +2518,9 @@ int *GfxCIDFont::getCodeToGIDMap(FoFiTrueType *ff, int *mapsizep) {
   return codeToGID;
 }
 
-double GfxCIDFont::getWidth (char* s, int len) {
-  int nUsed;
+double GfxCIDFont::getWidth(CID cid) {
   double w;
   int a, b, m;
-  CharCode c;
-
-  CID cid = cMap->getCID(s, len, &c, &nUsed);
 
   w = widths.defWidth;
   if (widths.nExceps > 0 && cid >= widths.exceps[0].first) {
@@ -2561,6 +2542,14 @@ double GfxCIDFont::getWidth (char* s, int len) {
   return w;
 }
 
+double GfxCIDFont::getWidth (char* s, int len) {
+  int nUsed;
+  CharCode c;
+
+  CID cid = cMap->getCID(s, len, &c, &nUsed);
+  return getWidth(cid);
+}
+
 //------------------------------------------------------------------------
 // GfxFontDict
 //------------------------------------------------------------------------
